package com.widget.imagevideobanner.banner;

import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.constraint.ConstraintLayout;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.VideoView;

import com.bumptech.glide.Glide;
import com.widget.imagevideobanner.R;
import com.widget.imagevideobanner.bean.BannerBean;
import com.widget.imagevideobanner.utils.BannerLogFileUtils;
import com.widget.imagevideobanner.utils.BitmapUtils;

import java.io.File;
import java.lang.ref.WeakReference;
import java.math.BigDecimal;


public class ImageVideoFragment extends Fragment {

    private static final String TAG = ImageVideoFragment.class.getSimpleName();
    private OnVideoCompletionListener listener;
    private VideoView mVideoView;
    private BannerBean bannerBean;
    // 视频是否循环播放
    private boolean loop;
    private ImageView ivWaitLoading;
    private ConstraintLayout cVideoView;
    private LinearLayout llWaitLoading;
    private int currentPosition;
    private boolean playerPaused;
    private String mUrl;
    private static final int STOP_PLAYER = 0x2000;
    private static final int START_PLAYER = 0x2001;
    private static final int PAUSE_PLAYER = 0x2002;
    private static final int SET_VIDEO_URL = 0x2003;
    private static final int PROGRESS = 0x2004;

    /**
     * 使用Handler是为了避免出现ANR异常
     */
    private Handler handler;


    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Bundle bundle = getArguments();
        bannerBean = (BannerBean) bundle.getSerializable("bannerBean");
        loop = (boolean) bundle.getBoolean("loop");
        Log.e(TAG, "type=" + bannerBean.getType() + ",url=" + bannerBean.getUrl());
        handler = new InnerHandler(this);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view;
        if (bannerBean != null) {
            int type = bannerBean.getType();
            if (type == 0) {
                view = LayoutInflater.from(getActivity()).inflate(R.layout.item_image_view, container, false);
                ImageView imageView = view.findViewById(R.id.iv);
                File file = new File(bannerBean.getUrl());
                if (file.exists()) {
                    Glide.with(container.getContext()).load(file)
                            .into(imageView);
                } else {
                    Glide.with(container.getContext()).load(bannerBean.getUrl())
                            .into(imageView);
                }

            } else {
                view = LayoutInflater.from(getActivity()).inflate(R.layout.item_video_view, container, false);
                mVideoView = view.findViewById(R.id.video_view);
                ivWaitLoading = view.findViewById(R.id.iv_wait_loading_layout);
                llWaitLoading = view.findViewById(R.id.wait_loading_layout);
                cVideoView = view.findViewById(R.id.cVideoView);
                setVideoBackground();
                setPicture(bannerBean.getWidth(),bannerBean.getHeight());
                initData();
            }
        } else {
            view = LayoutInflater.from(getActivity()).inflate(R.layout.item_image_view, container, false);
        }
        return view;
    }

    public void setVideoBackground(){
        if (bannerBean.getLoadingImage() == null) {
            cVideoView.setBackgroundResource(R.drawable.guide_defaut);
        } else {
            cVideoView.setBackground(BitmapUtils.bitmap2Drawable(BitmapUtils.getBitmap(bannerBean.getLoadingBlurImage())));
        }
    }

    /**
     * 设置加载视频的等待图片
     */
    public void setPicture() {
        //视频未加载完成，使用菊花
        if (bannerBean.getLoadingImage() == null) {
            if (bannerBean.getUrl().contains("android.resource")) {
                ivWaitLoading.setVisibility(View.VISIBLE);
                ivWaitLoading.setImageResource(R.drawable.guide_defaut);
                llWaitLoading.setVisibility(View.GONE);
            } else {
                ivWaitLoading.setVisibility(View.GONE);
                llWaitLoading.setVisibility(View.VISIBLE);
            }
        } else {//加载完成使用首帧图片
            ivWaitLoading.setVisibility(View.VISIBLE);
            ivWaitLoading.setImageBitmap(BitmapUtils.getBitmap(bannerBean.getLoadingImage()));
            llWaitLoading.setVisibility(View.GONE);
        }
    }

    /**
     * 设置加载视频的等待图片
     */
    public void setPicture(int width,int height) {

        if(width == 0 || height == 0){
            return;
        }

        Log.e("####","草"+width+"  "+height);
        //设置设置尺寸
        ConstraintLayout.LayoutParams lp2 = (ConstraintLayout.LayoutParams) ivWaitLoading.getLayoutParams();
        lp2.height = height;
        lp2.width = width;
        ivWaitLoading.setLayoutParams(lp2);

        //视频未加载完成，使用菊花
        if (bannerBean.getLoadingImage() == null) {
            if (bannerBean.getUrl().contains("android.resource")) {
                ivWaitLoading.setVisibility(View.VISIBLE);
                ivWaitLoading.setImageResource(R.drawable.guide_defaut);
                llWaitLoading.setVisibility(View.GONE);
            } else {
                ivWaitLoading.setVisibility(View.GONE);
                llWaitLoading.setVisibility(View.VISIBLE);
            }
        } else {//加载完成使用首帧图片
            ivWaitLoading.setVisibility(View.VISIBLE);
            ivWaitLoading.setImageBitmap(BitmapUtils.getBitmap(bannerBean.getLoadingImage()));
            llWaitLoading.setVisibility(View.GONE);
        }
    }


    private void initData() {

        if (null != mVideoView) {
            mVideoView.setAlpha(0);
            mVideoView.requestFocus();
            try {
                mVideoView.setVideoURI(Uri.parse(bannerBean.getUrl()));
            } catch (Exception e) {

            }
            mVideoView.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {
                @Override
                public void onCompletion(MediaPlayer mp) {
//                    mVideoView.setBackgroundColor(Color.TRANSPARENT);
//                    mVideoView.setVisibility(View.GONE);
                    mVideoView.pause();
                    if (null != listener) {
                        listener.onVideoCompletion(mp);
                    }
                }
            });

            mVideoView.setOnPreparedListener(new MediaPlayer.OnPreparedListener() {
                @Override
                public void onPrepared(MediaPlayer mp) {

                    //重新刷新大小
                    refreshPortraitScreen(mp.getVideoWidth(),mp.getVideoHeight());

                    mp.setLooping(loop);
                    Log.e(TAG, "视频加载完成" + bannerBean.getUrl());
                    mp.setOnInfoListener(new MediaPlayer.OnInfoListener() {
                        @Override
                        public boolean onInfo(MediaPlayer mp, int what, int extra) {
                            if (what == MediaPlayer.MEDIA_INFO_VIDEO_RENDERING_START) {
                                mVideoView.setAlpha(1);
                                handler.sendEmptyMessage(PROGRESS);
                                llWaitLoading.setVisibility(View.GONE);
                                ivWaitLoading.setVisibility(View.GONE);
                                mVideoView.setBackgroundColor(Color.TRANSPARENT);
                                return true;
                            }
                            return false;
                        }
                    });
                }
            });

            mVideoView.setOnErrorListener(new MediaPlayer.OnErrorListener() {
                @Override
                public boolean onError(MediaPlayer mp, int what, int extra) {

                    BannerLogFileUtils.saveInfo2File("视频播放出错了-what=" + what + ",extra=" + extra);

                    Log.e(TAG, "视频播放出错了-what=" + what + ",extra=" + extra);
                    mVideoView.stopPlayback();
                    if (null != listener) {
                        listener.onError(mp);
                    }
                    if (what == MediaPlayer.MEDIA_ERROR_SERVER_DIED) {
                        //媒体服务器挂掉了。此时，程序必须释放MediaPlayer 对象，并重新new 一个新的。
                        Log.e(TAG, "媒体服务器挂掉了");
                    } else if (what == MediaPlayer.MEDIA_ERROR_UNKNOWN) {
                        if (extra == MediaPlayer.MEDIA_ERROR_IO) {
                            //文件不存在或错误，或网络不可访问错误
                            Log.e(TAG, "文件不存在或错误，或网络不可访问错误");
                        } else if (extra == MediaPlayer.MEDIA_ERROR_TIMED_OUT) {
                            //超时
                            Log.e(TAG, "超时");
                        }
                    }
                    return true;
                }
            });
            sendSetVideoUrlMsg();
        }
    }


    public void refreshPortraitScreen(int width,int height) {

        int mHeight = mVideoView.getHeight();
        int mWidth = mVideoView.getWidth();

        BigDecimal viewScale = new BigDecimal(mWidth+"")
                .divide(new BigDecimal(mHeight+""),2, BigDecimal.ROUND_HALF_UP);
        BigDecimal videoScale = new BigDecimal(width+"")
                .divide(new BigDecimal(height+""),2, BigDecimal.ROUND_HALF_UP);

        if(viewScale.compareTo(videoScale) >= 0){
            //按照高度缩放
            BigDecimal heightScale = new BigDecimal(mHeight+"")
                    .divide(new BigDecimal(height+""),2, BigDecimal.ROUND_HALF_UP);
            height = mHeight;
            width =  new BigDecimal(width+"").multiply(heightScale).intValue();
        }else{
            //按照宽度缩放
            BigDecimal widthScale = new BigDecimal(mWidth+"")
                    .divide(new BigDecimal(width+""),2, BigDecimal.ROUND_HALF_UP);
            width = mWidth;
            height =  new BigDecimal(height+"").multiply(widthScale).intValue();
        }

        //设置videoview 宽高
        ConstraintLayout.LayoutParams lp = (ConstraintLayout.LayoutParams) mVideoView.getLayoutParams();
        lp.height = height;
        lp.width = width;
        mVideoView.setLayoutParams(lp);

        if(bannerBean.getWidth() == 0 || bannerBean.getHeight() == 0){
            bannerBean.setWidth(width);
            bannerBean.setHeight(height);
            listener.updateSize(bannerBean);
        }
    }


    private void setVideoUrl() {

        String url = bannerBean.getUrl();
        mUrl = url;
        //播放本地视频
        try {
            mVideoView.setVideoURI(Uri.parse(url));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void startPlayer() {
        if (null != mVideoView) {
//            setWaitLoading();
            Log.e(TAG, "startPlayer " + currentPosition);
            mVideoView.setBackgroundColor(Color.TRANSPARENT);
            mVideoView.seekTo(currentPosition);
            mVideoView.start();
        }
    }

//    /**
//     * 截取当前进度的截图，作为等待画面
//     */
//    private void setWaitLoading() {
//        if(currentPosition != 0){
//            setPicture(currentPosition * 1000,bannerBean.getUrl());
//        }
//    }


    public void circulationPlayer() {
        /*if (null != mVideoView) {
            mVideoView.setVideoPath(bannerBean.getUrl());
            mVideoView.start();
        }*/
        sendStartVideoMsg(true);
    }

    private void stopPlayer() {
        Log.e(TAG, "stopPlayer");
        if (null != mVideoView) {
            mVideoView.stopPlayback();
            handler.removeCallbacksAndMessages(null);
        }
    }


    public boolean isPlaying() {
        if (null != mVideoView) {
            return mVideoView.isPlaying();
        }
        return false;
    }

    public int getCurrentPosition() {
        if (null != mVideoView) {
            return mVideoView.getCurrentPosition();
        }
        return 0;
    }


    private void pausePlayer() {
        if (null != mVideoView) {
            Log.e(TAG, "pausePlayer 当前进度是" + mVideoView.getCurrentPosition());
            mVideoView.setBackgroundColor(getResources().getColor(R.color.white));
            mVideoView.setAlpha(0);
            playerPaused = true;
            this.currentPosition = mVideoView.getCurrentPosition();
            mVideoView.pause();
        }
    }

    private void sendStartVideoMsg() {
        sendStartVideoMsg(false);
    }

    private void sendStartVideoMsg(boolean isHasUrl) {
        removeMessages();
        if (!handler.hasMessages(START_PLAYER)) {
            if (null != mVideoView) {
                if (isHasUrl) {
                    try {
                        mVideoView.setVideoURI(Uri.parse(mUrl));
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                handler.sendEmptyMessage(START_PLAYER);
            }
        }
    }

    private void sendStopVideoMsg() {
        removeMessages();
        if (!handler.hasMessages(STOP_PLAYER)) {
            if (null != mVideoView) {
                handler.sendEmptyMessage(STOP_PLAYER);
            }
        }
    }

    private void sendPauseVideoMsg() {
        removeMessages();
        if (!handler.hasMessages(PAUSE_PLAYER)) {
            if (null != mVideoView) {
                handler.sendEmptyMessage(PAUSE_PLAYER);
            }
        }
    }

    private void sendSetVideoUrlMsg() {
        removeMessages();
        if (!handler.hasMessages(SET_VIDEO_URL)) {
            if (null != mVideoView) {
                handler.sendEmptyMessage(SET_VIDEO_URL);
            }
        }
    }

    private void removeMessages() {
        if (handler.hasMessages(START_PLAYER)) {
            handler.removeMessages(START_PLAYER);
        }
        if (handler.hasMessages(STOP_PLAYER)) {
            handler.removeMessages(STOP_PLAYER);
        }
        if (handler.hasMessages(PAUSE_PLAYER)) {
            handler.removeMessages(PAUSE_PLAYER);
        }
        if (handler.hasMessages(SET_VIDEO_URL)) {
            handler.removeMessages(SET_VIDEO_URL);
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        if (playerPaused) {
//            startPlayer();
            sendStartVideoMsg();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
//        pausePlayer();
        sendPauseVideoMsg();
    }

    @Override
    public void onDestroyView() {
        //2021/2/24 记录：这里要判断LoadingImage的存在，否则会回收本地Resource，造成加载错误.本地id标识的图片也会处理为BitmapDrawable
        if (bannerBean.getLoadingImage() != null && ivWaitLoading != null) {
            Drawable drawable = ivWaitLoading.getDrawable();
            ivWaitLoading.setImageDrawable(null);
            if (drawable instanceof BitmapDrawable) {
                Bitmap bm = ((BitmapDrawable) drawable).getBitmap();
                if (!bm.isRecycled()) {
                    bm.recycle();
                }
            }
        }
        super.onDestroyView();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        //stopPlayer();
        sendStopVideoMsg();
        Log.e(TAG, "onDestroy=" + bannerBean.getUrl());
    }

    public interface OnVideoCompletionListener {
        void onVideoCompletion(MediaPlayer mp);
        void onError(MediaPlayer mp);
        void updateSize(BannerBean bannerBean);
    }

    public void setOnVideoCompletionListener(OnVideoCompletionListener listener) {
        this.listener = listener;
    }

    private static class InnerHandler extends Handler{
        private final WeakReference<Fragment> mFragment;
        public InnerHandler(Fragment fragment){
            mFragment =new WeakReference<Fragment>(fragment);
        }
        @Override
        public void handleMessage(Message msg) {
            ImageVideoFragment fragment=(ImageVideoFragment)mFragment.get();
            super.handleMessage(msg);
            if(fragment!=null){
                switch (msg.what) {
                    case STOP_PLAYER:
                        fragment.stopPlayer();
                        break;
                    case START_PLAYER:
                        fragment.startPlayer();
                        break;
                    case PAUSE_PLAYER:
                        fragment.pausePlayer();
                        break;
                    case SET_VIDEO_URL:
                        fragment.setVideoUrl();
                        fragment.startPlayer();
                        break;
                }
            }
        }
    }
}
